@@ -39,7 +39,9 @@ char copyright[] =
  * From: @(#)telnetd.c	5.48 (Berkeley) 3/1/91
  */
 char telnetd_rcsid[] = 
-  "$Id: telnetd.c,v 1.9 1996/12/29 18:16:33 dholland Exp $";
+  "$Id: telnetd.c,v 1.16 1999/08/19 01:05:40 dholland Exp $";
+
+#include "../version.h"
 
 #include <netdb.h>
 #include <termcap.h>
@@ -48,6 +50,7 @@ char telnetd_rcsid[] =
 #include <arpa/inet.h>
 #include "telnetd.h"
 #include "pathnames.h"
+#include "setproctitle.h"
 
 #if	defined(AUTHENTICATE)
 #include <libtelnet/auth.h>
@@ -71,11 +74,6 @@ char	ptyibuf2[BUFSIZ];
 
 int	hostinfo = 1;			/* do we print login banner? */
 
-#ifdef	CRAY
-extern int      newmap; /* nonzero if \n maps to ^M^J */
-int	lowpty = 0, highpty;	/* low, high pty numbers */
-#endif /* CRAY */
-
 int debug = 0;
 int keepalive = 1;
 char *loginprg = _PATH_LOGIN;
@@ -84,7 +82,7 @@ char *progname;
 extern void usage(void);
 
 int
-main(int argc, char *argv[])
+main(int argc, char *argv[], char *env[])
 {
 	struct sockaddr_in from;
 	int on = 1;
@@ -95,6 +93,8 @@ main(int argc, char *argv[])
 	int tos = -1;
 #endif
 
+	initsetproctitle(argc, argv, env);
+
 	pfrontp = pbackp = ptyobuf;
 	netip = netibuf;
 	nfrontp = nbackp = netobuf;
@@ -104,14 +104,6 @@ main(int argc, char *argv[])
 
 	progname = *argv;
 
-#ifdef CRAY
-	/*
-	 * Get number of pty's before trying to process options,
-	 * which may include changing pty range.
-	 */
-	highpty = getnpty();
-#endif /* CRAY */
-
 	while ((ch = getopt(argc, argv, "d:a:e:lhnr:I:D:B:sS:a:X:L:")) != EOF) {
 		switch(ch) {
 
@@ -196,15 +188,6 @@ main(int argc, char *argv[])
 			hostinfo = 0;
 			break;
 
-#if	defined(CRAY) && defined(NEWINIT)
-		case 'I':
-		    {
-			extern char *gen_id;
-			gen_id = optarg;
-			break;
-		    }
-#endif	/* defined(CRAY) && defined(NEWINIT) */
-
 #ifdef	LINEMODE
 		case 'l':
 			alwayslinemode = 1;
@@ -219,34 +202,6 @@ main(int argc, char *argv[])
 			keepalive = 0;
 			break;
 
-#ifdef CRAY
-		case 'r':
-		    {
-			char *strchr();
-			char *c;
-
-			/*
-			 * Allow the specification of alterations
-			 * to the pty search range.  It is legal to
-			 * specify only one, and not change the
-			 * other from its default.
-			 */
-			c = strchr(optarg, '-');
-			if (c) {
-				*c++ = '\0';
-				highpty = atoi(c);
-			}
-			if (*optarg != '\0')
-				lowpty = atoi(optarg);
-			if ((lowpty > highpty) || (lowpty < 0) ||
-							(highpty > 32767)) {
-				usage();
-				/* NOT REACHED */
-			}
-			break;
-		    }
-#endif	/* CRAY */
-
 #ifdef	SecurID
 		case 's':
 			/* SecurID required */
@@ -340,10 +295,6 @@ main(int argc, char *argv[])
 	    (void) dup2(ns, 0);
 	    (void) close(ns);
 	    (void) close(s);
-#ifdef convex
-	} else if (argc == 1) {
-		; /* VOID*/		/* Just ignore the host/port name */
-#endif
 	} else if (argc > 0) {
 		usage();
 		/* NOT REACHED */
@@ -400,17 +351,11 @@ usage(void)
 	fprintf(stderr, " [-edebug]");
 #endif
 	fprintf(stderr, " [-h]");
-#if	defined(CRAY) && defined(NEWINIT)
-	fprintf(stderr, " [-Iinitid]");
-#endif
 #ifdef LINEMODE
 	fprintf(stderr, " [-l]");
 #endif
 	fprintf(stderr, " [-L login_program]");
 	fprintf(stderr, " [-n]");
-#ifdef	CRAY
-	fprintf(stderr, " [-r[lowpty]-[highpty]]");
-#endif
 #ifdef	SecurID
 	fprintf(stderr, " [-s]");
 #endif
@@ -523,22 +468,35 @@ getterminaltype(char *name)
 	 * we have to just go with what we (might) have already gotten.
 	 */
 	if (his_state_is_will(TELOPT_TTYPE) && !terminaltypeok(terminaltype)) {
-	    (void) strncpy(first, terminaltype, sizeof(first));
+	    /*
+	     * Due to state.c, terminaltype points to a static char[41].
+	     * Therefore, this assert cannot fail, and therefore, strings
+	     * arising from "terminaltype" can be safely strcpy'd into
+	     * first[] or last[].
+	     */
+	    assert(strlen(terminaltype) < sizeof(first));
+
+	    strcpy(first, terminaltype);
+
 	    for(;;) {
 		/*
 		 * Save the unknown name, and request the next name.
 		 */
-		(void) strncpy(last, terminaltype, sizeof(last));
+		strcpy(last, terminaltype);
+
 		_gettermname();
+		assert(strlen(terminaltype) < sizeof(first));
+
 		if (terminaltypeok(terminaltype))
 		    break;
-		if ((strncmp(last, terminaltype, sizeof(last)) == 0) ||
+
+		if (!strcmp(last, terminaltype) ||
 		    his_state_is_wont(TELOPT_TTYPE)) {
 		    /*
 		     * We've hit the end.  If this is the same as
 		     * the first name, just go with it.
 		     */
-		    if (strncmp(first, terminaltype, sizeof(first)) == 0)
+		    if (!strcmp(first, terminaltype))
 			break;
 		    /*
 		     * Get the terminal name one more time, so that
@@ -546,8 +504,15 @@ getterminaltype(char *name)
 		     * the start of the list.
 		     */
 		     _gettermname();
-		    if (strncmp(first, terminaltype, sizeof(first)) != 0)
-			(void) strncpy(terminaltype, first, sizeof(first));
+		    assert(strlen(terminaltype) < sizeof(first));
+
+		    if (strcmp(first, terminaltype)) {
+			/*
+			 * first[] came from terminaltype, so it must fit
+			 * back in.
+			 */
+			strcpy(terminaltype, first);
+		    }
 		    break;
 		}
 	    }
@@ -582,6 +547,14 @@ terminaltypeok(char *s)
     if (terminaltype == NULL)
 	return(1);
 
+    /*
+     * Fix from Chris Evans: if it has a / in it, termcap will
+     * treat it as a filename. Oops.
+     */
+    if (strchr(s, '/')) {
+	return 0;
+    }
+
     /*
      * tgetent() will return 1 if the type is known, and
      * 0 if it is not known.  If it returns -1, it couldn't
@@ -601,11 +574,7 @@ terminaltypeok(char *s)
 char host_name[MAXHOSTNAMELEN];
 char remote_host_name[MAXHOSTNAMELEN];
 
-#ifndef	convex
 extern void telnet(int, int);
-#else
-extern void telnet(int, int, char *);
-#endif
 
 /*
  * Get a pty, scan input lines.
@@ -621,25 +590,9 @@ doit(struct sockaddr_in *who)
 	/*
 	 * Find an available pty to use.
 	 */
-#ifndef	convex
 	pty = getpty();
 	if (pty < 0)
 		fatal(net, "All network ports in use");
-#else
-	for (;;) {
-		char *lp;
-		extern char *line, *getpty();
-
-		if ((lp = getpty()) == NULL)
-			fatal(net, "Out of ptys");
-
-		if ((pty = open(lp, 2)) >= 0) {
-			strcpy(line,lp);
-			line[5] = 't';
-			break;
-		}
-	}
-#endif
 
 	/* get name of connected client */
 	hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
@@ -648,14 +601,25 @@ doit(struct sockaddr_in *who)
 		host = hp->h_name;
 	else
 		host = inet_ntoa(who->sin_addr);
+
 	/*
 	 * We must make a copy because Kerberos is probably going
 	 * to also do a gethost* and overwrite the static data...
 	 */
-	strncpy(remote_host_name, host, sizeof(remote_host_name)-1);
-	remote_host_name[sizeof(remote_host_name)-1] = 0;
+	{
+		int i;
+		strncpy(remote_host_name, host, sizeof(remote_host_name)-1);
+		remote_host_name[sizeof(remote_host_name)-1] = 0;
+
+		/* Disallow funnies. */
+		for (i=0; remote_host_name[i]; i++) {
+		    if (remote_host_name[i]<=32 || remote_host_name[i]>126) 
+			remote_host_name[i] = '?';
+		}
+	}
 	host = remote_host_name;
 
+	/* Get local host name */
 	{
 		struct hostent *h;
 		gethostname(host_name, sizeof(host_name));
@@ -680,46 +644,27 @@ doit(struct sockaddr_in *who)
 
 	/* TODO list stuff provided by Laszlo Vecsey <master@internexus.net> */
 
-	sprintf(progname, "telnetd: %s [%s]", host, terminaltype);
+	/*
+	 * Set REMOTEHOST environment variable
+	 */
+	setproctitle("%s", host);
 	setenv("REMOTEHOST", host, 0);
 
 	/*
 	 * Start up the login process on the slave side of the terminal
 	 */
-#ifndef	convex
 	startslave(host, level, user_name);
 
 	telnet(net, pty);  /* begin server processing */
-#else
-	telnet(net, pty, host);
-#endif
+
 	/*NOTREACHED*/
 }  /* end of doit */
 
-#if	defined(CRAY2) && defined(UNICOS5) && defined(UNICOS50)
-	int
-Xterm_output(ibufp, obuf, icountp, ocount)
-	char **ibufp, *obuf;
-	int *icountp, ocount;
-{
-	int ret;
-	ret = term_output(*ibufp, obuf, *icountp, ocount);
-	*ibufp += *icountp;
-	*icountp = 0;
-	return(ret);
-}
-#define	term_output	Xterm_output
-#endif	/* defined(CRAY2) && defined(UNICOS5) && defined(UNICOS50) */
-
 /*
  * Main loop.  Select from pty and network, and
  * hand data to telnet receiver finite state machine.
  */
-#ifndef	convex
 void telnet(int f, int p)
-#else
-void telnet(int f, int p, char *host)
-#endif
 {
     int on = 1;
     char *HE;
@@ -846,9 +791,6 @@ void telnet(int f, int p, char *host)
     
     ioctl(f, FIONBIO, (char *)&on);
     ioctl(p, FIONBIO, (char *)&on);
-#if defined(CRAY2) && defined(UNICOS5)
-    init_termdriver(f, p, interrupt, sendbrk);
-#endif
 
 #if defined(SO_OOBINLINE)
     setsockopt(net, SOL_SOCKET, SO_OOBINLINE, &on, sizeof on);
@@ -867,21 +809,6 @@ void telnet(int f, int p, char *host)
     
     signal(SIGCHLD, cleanup);
     
-#if defined(CRAY2) && defined(UNICOS5)
-    /*
-     * Cray-2 will send a signal when pty modes are changed by slave
-     * side.  Set up signal handler now.
-     */
-    if ((int)signal(SIGUSR1, termstat) < 0)
-	perror("signal");
-    else if (ioctl(p, TCSIGME, (char *)SIGUSR1) < 0)
-	perror("ioctl:TCSIGME");
-    /*
-     * Make processing loop check terminal characteristics early on.
-     */
-    termstat();
-#endif
-
 #ifdef TIOCNOTTY
     {
 	register int t;
@@ -893,11 +820,6 @@ void telnet(int f, int p, char *host)
     }
 #endif
     
-#if defined(CRAY) && defined(NEWINIT) && defined(TIOCSCTTY)
-    setsid();
-    ioctl(p, TIOCSCTTY, 0);
-#endif
-
     /*
      * Show banner that getty never gave.
      *
@@ -906,10 +828,8 @@ void telnet(int f, int p, char *host)
      * other pty --> client data.
      */
     
-#if !defined(CRAY) || !defined(NEWINIT)
     if (getenv("USER"))
 	hostinfo = 0;
-#endif
     
     IM = DEFAULT_IM;
     HE = 0;
@@ -933,10 +853,6 @@ void telnet(int f, int p, char *host)
 	 {sprintf(nfrontp, "td: Entering processing loop\r\n");
 	 nfrontp += strlen(nfrontp);});
     
-#ifdef	convex
-    startslave(host);
-#endif
-
     for (;;) {
 	fd_set ibits, obits, xbits;
 	int c, hifd;
@@ -944,9 +860,6 @@ void telnet(int f, int p, char *host)
 	if (ncc < 0 && pcc < 0)
 	    break;
 	
-#if	defined(CRAY2) && defined(UNICOS5)
-	if (needtermstat) _termstat();
-#endif	/* defined(CRAY2) && defined(UNICOS5) */
 	FD_ZERO(&ibits);
 	FD_ZERO(&obits);
 	FD_ZERO(&xbits);
@@ -1086,7 +999,6 @@ void telnet(int f, int p, char *host)
 	    else {
 		if (pcc <= 0)
 		    break;
-#if	!defined(CRAY2) || !defined(UNICOS5)
 #ifdef	LINEMODE
 				/*
 				 * If ioctl from pty, pass it through net
@@ -1122,16 +1034,6 @@ void telnet(int f, int p, char *host)
 		}
 		pcc--;
 		ptyip = ptyibuf+1;
-#else	/* defined(CRAY2) && defined(UNICOS5) */
-		if (!uselinemode) {
-		    unpcc = pcc;
-		    unptyip = ptyibuf;
-		    pcc = term_output(&unptyip, ptyibuf2,
-				      &unpcc, BUFSIZ);
-		    ptyip = ptyibuf2;
-		} 
-		else ptyip = ptyibuf;
-#endif	/* defined(CRAY2) && defined(UNICOS5) */
 	    }
 	}
 	
@@ -1141,11 +1043,6 @@ void telnet(int f, int p, char *host)
 	    c = *ptyip++ & 0377, pcc--;
 	    if (c == IAC)
 		*nfrontp++ = c;
-#if defined(CRAY2) && defined(UNICOS5)
-	    else if (c == '\n' &&
-		     my_state_is_wont(TELOPT_BINARY) && newmap)
-		*nfrontp++ = '\r';
-#endif	/* defined(CRAY2) && defined(UNICOS5) */
 	    *nfrontp++ = c;
 	    if ((c == '\r'  ) && (my_state_is_wont(TELOPT_BINARY))) {
 		if (pcc > 0 && ((*ptyip & 0377) == '\n')) {
@@ -1155,17 +1052,6 @@ void telnet(int f, int p, char *host)
 		else *nfrontp++ = '\0';
 	    }
 	}
-#if defined(CRAY2) && defined(UNICOS5)
-	/*
-	 * If chars were left over from the terminal driver,
-	 * note their existence.
-	 */
-	if (!uselinemode && unpcc) {
-	    pcc = unpcc;
-	    unpcc = 0;
-	    ptyip = unptyip;
-	}
-#endif	/* defined(CRAY2) && defined(UNICOS5) */
 
 	if (FD_ISSET(f, &obits) && (nfrontp - nbackp) > 0)
 	    netflush();
@@ -1246,7 +1132,7 @@ void recv_ayt(void) {
 void doeof(void) {
     init_termbuf();
 
-#if	defined(LINEMODE) && defined(USE_TERMIO) && (VEOF == VMIN)
+#if	defined(LINEMODE) && (VEOF == VMIN)
     if (!tty_isediting()) {
 	extern char oldeofc;
 	*pfrontp++ = oldeofc;
